package nl.iobyte.themepark.listener;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.events.ridecount.ChangeCountEvent;
import nl.iobyte.themepark.api.ridecount.CountManager;
import nl.iobyte.themepark.api.utils.ItemBuilder;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.menu.AttractionMenu;
import nl.iobyte.themepark.ridecount.RideCountAPI;
import nl.iobyte.themepark.util.Color;
import nl.iobyte.themepark.util.MessageUtil;
import nl.iobyte.workchain.components.Work;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.*;
import org.bukkit.inventory.ItemStack;
import java.util.*;
import java.util.concurrent.CompletableFuture;

public class PlayerListener implements Listener {

    private final boolean enabled, clear;
    private final ItemStack menuItem;
    private final List<String> worlds;
    private final int slot;

    public PlayerListener() {
        YamlConfig settings = ThemeParkPlugin.getInstance().getSettings();

        enabled = settings.getConfig().getBoolean("item.enabled");
        clear = settings.getConfig().getBoolean("item.clear");
        worlds = settings.getConfig().getStringList("item.worlds");
        slot = settings.getConfig().getInt("item.slot");

        Material material = Material.getMaterial(settings.getConfig().getString("item.material"));
        short data = Short.parseShort(settings.getConfig().getString("item.data"));
        String name = Color.color(settings.getConfig().getString("item.display-name"));
        if(material != null && !name.isEmpty()) {
            ItemBuilder builder = new ItemBuilder(material, 1, data);
            builder.setName(name);
            menuItem = builder.getItem();
        } else {
            menuItem = null;
        }
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent e) {
        Player player = e.getPlayer();
        UUID uuid = player.getUniqueId();

        doItem(e.getPlayer());
        loadRideCount(uuid);
    }

    @EventHandler
    public void onSwitch(PlayerChangedWorldEvent e) {
        doItem(e.getPlayer());
    }

    public void doItem(Player player) {
        YamlConfig config = ThemeParkPlugin.getInstance().getData();

        CompletableFuture<ItemStack> fc = new CompletableFuture<>();
        Work.firstTask(() -> {
            if(!enabled)
                return null;

            boolean b = true;
            if (config.getConfig().contains(player.getUniqueId().toString() + ".item")) {
                b = config.getConfig().getBoolean(player.getUniqueId().toString() + ".item");
            } else {
                config.getConfig().set(player.getUniqueId().toString() + ".item", true);
                config.save();
            }

            return b;
        }).abortIfNull(() -> {
            fc.complete(null);
        }).nextTask(b -> {
            if(!b)
                return null;

            return menuItem;
        }).abortIfNull(() -> {
            fc.complete(null);
        }).lastTask(item -> {
            if(clear)
                player.getInventory().clear();

            if(worlds != null && !worlds.isEmpty()) {
                if (worlds.contains(player.getLocation().getWorld().getName())) {
                    fc.complete(new ItemStack(Material.AIR));
                    return;
                }
            }

            fc.complete(item);
        }).setFullExecute(true).execute();

        fc.thenAccept(item -> {
            if(item == null)
                return;

            player.getInventory().setItem(slot, item);
            player.updateInventory();
        });
    }

    @EventHandler
    public void onInteract(PlayerInteractEvent e) {
        YamlConfig config = ThemeParkPlugin.getInstance().getSettings();
        Material material = Material.getMaterial(config.getConfig().getString("item.material"));
        String name = Color.color(config.getConfig().getString("item.display-name"));
        if(e.getItem() == null)
            return;

        ItemStack item = e.getItem();
        if(item.getType() != material)
            return;

        if(!item.hasItemMeta())
            return;

        if(!item.getItemMeta().getDisplayName().equals(name))
            return;

        if(e.getAction() == Action.LEFT_CLICK_AIR ||
                e.getAction() == Action.LEFT_CLICK_BLOCK)
            return;

        e.setCancelled(true);
        Player player = e.getPlayer();
        AttractionMenu.open(player);
    }

    @EventHandler
    public void onDrop(PlayerDropItemEvent e) {
        if(menuItem == null)
            return;

        Player player = e.getPlayer();
        if(player.hasPermission("themepark.admin"))
            return;

        ItemStack item = e.getItemDrop().getItemStack();
        if(item.getType() != menuItem.getType())
            return;

        if(!item.hasItemMeta())
            return;

        if(item.getItemMeta().getDisplayName().equals(menuItem.getItemMeta().getDisplayName()))
            return;

        e.setCancelled(true);
    }

    @EventHandler
    public void onQuit(PlayerQuitEvent e) {
        Player player = e.getPlayer();
        UUID uuid = player.getUniqueId();

        CountManager.removeCount(uuid);
    }

    private void loadRideCount(UUID uuid) {
        Work.firstTask(() -> {
            for(Attraction attraction : API.getAttractions().values())
                RideCountAPI.getCount(uuid, attraction).thenAccept(count -> Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> CountManager.setCount(attraction, uuid, count)));

            return true;
        }).abortIfNull().lastTask(result -> {}).setFullExecute(true).execute();
    }

    @EventHandler
    public void onChange(ChangeCountEvent e) {
        Player player = Bukkit.getPlayer(e.getUUID());
        if(player == null || !player.isOnline())
            return;

        Attraction attraction = e.getAttraction();
        int amount = e.getAfter() - e.getBefore();
        String message = MessageUtil.getMessage("ridecount.message");
        message = message.replace("{name}", attraction.getName());
        message = message.replace("{amount}", Integer.toString(amount));
        message = message.replace("{total}", Integer.toString(e.getCounter().getCount(e.getUUID()).getCount()));
        player.sendMessage(Color.color(message));
    }

}
